@@ -11,6 +11,8 @@ module Bundler
 
     attr_accessor :dependency_names
 
+    attr_reader :checksum_store
+
     def unmet_deps
       specs.unmet_dependency_names
     end
@@ -33,6 +35,8 @@ module Bundler
       spec.source == self
     end
 
+    def prefer_local!; end
+
     def local!; end
 
     def local_only!; end
