@@ -16,11 +16,11 @@ class RDoc::Markup::BlankLine
   ##
   # Calls #accept_blank_line on +visitor+
 
-  def accept visitor
+  def accept(visitor)
     visitor.accept_blank_line self
   end
 
-  def pretty_print q # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.text 'blankline'
   end
 
