@@ -7,11 +7,11 @@ class RDoc::Markup::Rule < Struct.new :weight
   ##
   # Calls #accept_rule on +visitor+
 
-  def accept visitor
+  def accept(visitor)
     visitor.accept_rule self
   end
 
-  def pretty_print q # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.group 2, '[rule:', ']' do
       q.pp weight
     end
